clear;
% Part 0: Preliminaries
% ======================================================================
% Parameters describing geometry:
% Centre of obstacle is (0,delta)
delta = 0.3;
% Radius of obstacle is R:
R = 0.5;
% Coordinates of centre of obstacle:
cx = 0;
cy = delta;
% Radii of obstacle (ax = ay means a circle, otherwise an ellipse):
ax = 0.7*R;
ay = R;
% Inner domain is from x=0 to x=ell, minus the half of the
% obstacle which lies in this region. Symmetry x <--> -x is
% assumed for the geometry. y always lies between -1 and +1.
ell = 1;

%%%
%%% geometry
%%%

% Use of geometry matrix:
% %%%%%%%%%%%%%%%%%%%%%%%
%
% Each column describes a curve in R^2.
%
% A column starting with a 2 describes a straight line; a column starting
% with a 4 describes an elliptical arc. In each case, the curve in question
% goes from (g(2,*),g(4,*)) to (g(3,*),g(5,*)).
%
% The equation of an elliptical arc is
%
% ((x-g(8,*))/g(10,*))^2 + ((y-g(9,*))/g(11,*))^2 = 1.
% 
% The entries in rows 6 and 7 describe the orientation.
%
g = [...
    2.0000    2.0000    2.0000    2.0000     2.0000    4.0000     4.0000;...
         0       ell       ell         0          0    0.0000      cx+ax;...
       ell       ell         0         0          0     cx+ax     0.0000;...
   -1.0000   -1.0000    1.0000    1.0000      cy-ay     cy-ay          0;...
   -1.0000    1.0000    1.0000     cy+ay    -1.0000         0      cy+ay;...
    1.0000    1.0000    1.0000    1.0000     1.0000         0          0;...
         0         0         0         0          0    1.0000     1.0000;...
         0         0         0         0          0        cx         cx;...
         0         0         0         0          0        cy         cy;...
         0         0         0         0          0        ax         ax;...
         0         0         0         0          0        ay         ay;...
         0         0         0         0          0         0          0];

npieces=size(g,2);


% Part 1: Neumann expansion truncation
% ======================================================================
% Inner domain eigenvalues up to lam_max will be used for the
% eigenfunction expansion of the inner ND map:

lam_max = 100;

% The value of N at which the Neumann expansion is truncated
% will be the number of Neumann eigenvalues below lam_max.

% Part 2: Neumann eigenvalue/eigenfunction calculation
% ======================================================================
%
% First set up mesh
% 
% nrefine = no. of mesh refinements
nrefine=1;
[p,e,t]=initmesh(g);
for k = 1:nrefine-1
[p,e,t]=refinemesh(g,p,e,t);
end;
[p1,e1,t1]=refinemesh(g,p,e,t);
% Plot mesh
figure(1), clf, pdemesh(p,e,t), axis equal, title('Uniform mesh');

% Neumann boundary condition vector:
b=bneum_0(7);

% Get Neumann eigenvalues and eigenfunctions:
[v,neum_eigs]=pdeeig(b,p1,e1,t1,1.0,0.,1.0,[-1 lam_max]);
N_eigs=length(neum_eigs);

% Get mass matrix for scaling of eigenfunctions:
[K1,M1,F1]=assema(p1,t1,0.,1.,0.);
%
% Scale Neumann eigenfunctions to have L_2 norms equal to 1:
for m=1:N_eigs
   v(:,m)=(v(:,m)'*M1*v(:,m))^(-1/2)*v(:,m);
end;

% Part 3: boundary eigenfunction basis set dimension. 
% ======================================================================
% Guess this!
 Nmax = 20;
% Remark: this is denoted M, not Nmax, in the paper.

% Part 4: interface eigenvalues and eigenfunctions. 
% ======================================================================
% We don't need to do these numerically for this simple problem.
% We have: w_n(y) = cos((n-1)*pi*(y+1)/2), y in (-1,1), n = 1,2,3,...,
% with eigenvalues \kappa_n = ((n-1)*pi/2)^2. However we do need to
% evaluate these functions at the boundary points, so let's do this.

% Find the indices of those mesh points which lie on the boundary:
p_indices=find(p1(1,:)==ell);
[y_values, ix]=sort(p1(2, p_indices));
%

% Find the values of the interface eigenfunctions at the boundary
% mesh points:
Psi=zeros(length(p_indices), Nmax);
Psi(:,1)=sqrt(0.5);
for n=2:Nmax
   Psi(:,n)=cos((n-1)*pi/2*(y_values+1)');
end;


% Part 5: calculate the numbers S_{nm} 
% ======================================================================
% Initialize array:
Ss=zeros(N_eigs,Nmax);
% 
for m=1:N_eigs
% Get the trace of the m-th eigenfunction on the boundary
% (i.e. its values at the boundary mesh points):
   v_b=v(p_indices,m)';
% Form the inner product with the n-th interface eigenfunction:
   for n=1:Nmax
      Ss(m,n)=trapz(y_values', v_b(ix)'.*Psi(:,n));
	end;
end;

% Part 6: calculate the matrices D(\lambda) and T(\lambda)
% ======================================================================
% The expression for D is
%
% D(\lambda) = diag(1./(neum_eigs-\lambda));
%
% the expression for T is
%
% T(\lambda) = diag(-1./sqrt(\kappa-\lambda));
%
% where \kappa_n = ((n-1)*pi/2)^2.
% 
% We will use this later.

% Part 7: set the cutoff J
% ======================================================================
% 
 lam_range = 0; % Because lam_range=0, we are seeking resonances
% 
% Remark: lam_range is J
%

% Part 8': calculate the matrix R(\lambda) at some fixed lam_0:
% ======================================================================
lam_0 = -1;
for n = 1:Nmax
% Solve PDE with nth basis function as Neumann data:
    b = bmixed(2,[1, 3, 4, 5, 6, 7],n);
    u(:,n) = assempde(b,p1,e1,t1,1.0,-lam_0,0.0);
end;
% Recover solution on boundary and take innner product with
% interface eigenfunctions:
for n = 1:Nmax
    ubound = u(p_indices,n)';
    for m = 1:Nmax
        ND1(n,m) = trapz(y_values',ubound(ix)'.*Psi(:,m));
    end;
end;
% Remark: ND1 is R(\lam_0). We also calculate D(\lam_0):
DI = Ss'*inv(diag(neum_eigs-lam_0))*Ss;

% Part 9: calculate the Dirichlet eigenvalues:
% ======================================================================
b=bdir_0(7);
[v,dir_eigs]=pdeeig(b,p1,e1,t1,1.0,0.,1.0,[-1 lam_max]);


% Part 10: do plot of the condition number of
%          R(\lambda)-T(\lambda)^{-1} to identify eigenvalues
% ====================================================================
lam_Npoints = 500; % Resolution for the plot
lmin = neum_eigs(2)+0.07;
lmax = neum_eigs(3)-0.07;
[N_eig, n_m]= size(Ss);
alphas=pi^2/4*(0:1:(n_m-1)).^2;
lbound = lmin; 
ubound = lmax; 

% Set up the grid of lambda values for the condition number plot
lams=linspace(lbound, ubound, lam_Npoints+2);
lams=lams(2:end-1);
lamsx = linspace(lbound,ubound,lam_Npoints+2);
for k=1:length(lamsx)
                lam = lamsx(k); 
% Diagonal matrix T(\lambda):
                T=diag(-1./sqrt(alphas-lam));
% Diagonal matrix D:
                DL = diag(1./((neum_eigs-lam).*(neum_eigs-lam_0)));
% Calculate Rr(lam) = R(lam)T(\lam)^{-1} using the improved formula:

                ND1trunc = ND1(lam_range+1:n_m,lam_range+1:n_m);
                Sstrunc = Ss(1:N_eig,lam_range+1:n_m);
                Ttrunc = T(lam_range+1:n_m,lam_range+1:n_m);
                Rtrunc = ND1trunc+(lam-lam_0)*Sstrunc'*DL*Sstrunc;               
                rconds(k)=cond(Rtrunc-Ttrunc);
end; 

% Find candidate eigenvalue:
rcmax = 0.5;
lam_eig = lmin;
for k = 1:length(lamsx)
 if (rconds(k)>rcmax)
     imax = k; % Records position of point with maximum condition number
     lam_eig=lamsx(imax); % Records best approximation to eigenvalue so far
     rcmax = rconds(k);
 end;
end;

% Plot:
plot(lamsx,rconds); 
 for k = 1:length(neum_eigs)
     text(neum_eigs(k),0.8*rcmax,'N')
 end;
 xlabel('N denotes an inner Neumann eigenvalue; other maxima may be resonances/eigenvalues')
 
% Part 11': check the orthogonality condition if lam_range > 0.
% =========================================================================
if (lam_range>0)
    lam = lam_eig;
    T=diag(-1./sqrt(alphas-lam));
    DL = diag(1./((neum_eigs-lam).*(neum_eigs-lam_0)));
    ND1trunc = ND1(lam_range+1:n_m,lam_range+1:n_m);
    Sstrunc = Ss(1:N_eig,lam_range+1:n_m);
    Ttrunc = T(lam_range+1:n_m,lam_range+1:n_m);
    Rtrunc = ND1trunc+(lam-lam_0)*Sstrunc'*DL*Sstrunc;               
% Get eigenvector of Rtrunc with eigenvalue closest to zero:
    [c,el] = eig(Rtrunc);
    rmin = abs(el(1,1));
    imin = 1;
    for k = 1:size(el,1)
        ell(k) = el(k,k);
        if (abs(ell(k))<rmin) 
            imin = k;
            rmin = abs(ell(k));
        end;
    end;
% Corresponding eigenvector:
    Cvec = c(:,imin);
    Cvec = Cvec/norm(Cvec,2);
    Ssstrunc = Ss(1:N_eig,1:lam_range);
% Orthocheck should be close to zero for an eigenvalue,
% but it will also be close to zero if lam_eig is close
% to a resonance.
    disp('Eigenvalue approximation')
    lam_eig
    disp('Orthogonality check:')
    orthocheck = Ssstrunc'*DL*Sstrunc*Cvec
end;
